@@ -14,21 +14,21 @@
 
 """Unit tests for the Flask utilities"""
 
-import httplib2
+import datetime
 import json
-import unittest
+import logging
 
 import flask
-import six.moves.http_client as httplib
+import httplib2
 import mock
+import six.moves.http_client as httplib
 import six.moves.urllib.parse as urlparse
+import unittest2
 
-from oauth2client import GOOGLE_AUTH_URI
-from oauth2client import GOOGLE_TOKEN_URI
+import oauth2client
+from oauth2client import client
 from oauth2client import clientsecrets
-from oauth2client.contrib.flask_util import _get_flow_for_token
-from oauth2client.contrib.flask_util import UserOAuth2 as FlaskOAuth2
-from oauth2client.client import OAuth2Credentials
+from oauth2client.contrib import flask_util
 
 
 __author__ = 'jonwayne@google.com (Jon Wayne Parrott)'
@@ -64,25 +64,26 @@ class Http2Mock(object):
         return self
 
 
-class FlaskOAuth2Tests(unittest.TestCase):
+class FlaskOAuth2Tests(unittest2.TestCase):
 
     def setUp(self):
         self.app = flask.Flask(__name__)
         self.app.testing = True
         self.app.config['SECRET_KEY'] = 'notasecert'
-        self.oauth2 = FlaskOAuth2(
+        self.app.logger.setLevel(logging.CRITICAL)
+        self.oauth2 = flask_util.UserOAuth2(
             self.app,
             client_id='client_idz',
             client_secret='client_secretz')
 
     def _generate_credentials(self, scopes=None):
-        return OAuth2Credentials(
+        return client.OAuth2Credentials(
             'access_tokenz',
             'client_idz',
             'client_secretz',
             'refresh_tokenz',
-            '3600',
-            GOOGLE_TOKEN_URI,
+            datetime.datetime.utcnow() + datetime.timedelta(seconds=3600),
+            oauth2client.GOOGLE_TOKEN_URI,
             'Test',
             id_token={
                 'sub': '123',
@@ -91,7 +92,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
             scopes=scopes)
 
     def test_explicit_configuration(self):
-        oauth2 = FlaskOAuth2(
+        oauth2 = flask_util.UserOAuth2(
             flask.Flask(__name__), client_id='id', client_secret='secret')
 
         self.assertEqual(oauth2.client_id, 'id')
@@ -104,7 +105,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
         with mock.patch('oauth2client.clientsecrets.loadfile',
                         return_value=return_val):
 
-            oauth2 = FlaskOAuth2(
+            oauth2 = flask_util.UserOAuth2(
                 flask.Flask(__name__), client_secrets_file='file.json')
 
             self.assertEqual(oauth2.client_id, 'id')
@@ -112,19 +113,19 @@ class FlaskOAuth2Tests(unittest.TestCase):
 
     def test_delayed_configuration(self):
         app = flask.Flask(__name__)
-        oauth2 = FlaskOAuth2()
+        oauth2 = flask_util.UserOAuth2()
         oauth2.init_app(app, client_id='id', client_secret='secret')
         self.assertEqual(oauth2.app, app)
 
     def test_explicit_storage(self):
         storage_mock = mock.Mock()
-        oauth2 = FlaskOAuth2(
+        oauth2 = flask_util.UserOAuth2(
             flask.Flask(__name__), storage=storage_mock, client_id='id',
             client_secret='secret')
         self.assertEqual(oauth2.storage, storage_mock)
 
     def test_explicit_scopes(self):
-        oauth2 = FlaskOAuth2(
+        oauth2 = flask_util.UserOAuth2(
             flask.Flask(__name__), scopes=['1', '2'], client_id='id',
             client_secret='secret')
         self.assertEqual(oauth2.scopes, ['1', '2'])
@@ -136,17 +137,16 @@ class FlaskOAuth2Tests(unittest.TestCase):
 
         with mock.patch('oauth2client.clientsecrets.loadfile',
                         return_value=return_val):
-            self.assertRaises(
-                ValueError,
-                FlaskOAuth2,
-                flask.Flask(__name__), client_secrets_file='file.json')
+            with self.assertRaises(ValueError):
+                flask_util.UserOAuth2(flask.Flask(__name__),
+                                      client_secrets_file='file.json')
 
     def test_app_configuration(self):
         app = flask.Flask(__name__)
         app.config['GOOGLE_OAUTH2_CLIENT_ID'] = 'id'
         app.config['GOOGLE_OAUTH2_CLIENT_SECRET'] = 'secret'
 
-        oauth2 = FlaskOAuth2(app)
+        oauth2 = flask_util.UserOAuth2(app)
 
         self.assertEqual(oauth2.client_id, 'id')
         self.assertEqual(oauth2.client_secret, 'secret')
@@ -160,28 +160,26 @@ class FlaskOAuth2Tests(unittest.TestCase):
 
             app = flask.Flask(__name__)
             app.config['GOOGLE_OAUTH2_CLIENT_SECRETS_FILE'] = 'file.json'
-            oauth2 = FlaskOAuth2(app)
+            oauth2 = flask_util.UserOAuth2(app)
 
             self.assertEqual(oauth2.client_id, 'id2')
             self.assertEqual(oauth2.client_secret, 'secret2')
 
     def test_no_configuration(self):
-        self.assertRaises(
-            ValueError,
-            FlaskOAuth2,
-            flask.Flask(__name__))
+        with self.assertRaises(ValueError):
+            flask_util.UserOAuth2(flask.Flask(__name__))
 
     def test_create_flow(self):
         with self.app.test_request_context():
             flow = self.oauth2._make_flow()
             state = json.loads(flow.params['state'])
-            self.assertTrue('google_oauth2_csrf_token' in flask.session)
+            self.assertIn('google_oauth2_csrf_token', flask.session)
             self.assertEqual(
                 flask.session['google_oauth2_csrf_token'], state['csrf_token'])
             self.assertEqual(flow.client_id, self.oauth2.client_id)
             self.assertEqual(flow.client_secret, self.oauth2.client_secret)
-            self.assertTrue('http' in flow.redirect_uri)
-            self.assertTrue('oauth2callback' in flow.redirect_uri)
+            self.assertIn('http', flow.redirect_uri)
+            self.assertIn('oauth2callback', flow.redirect_uri)
 
             flow = self.oauth2._make_flow(return_url='/return_url')
             state = json.loads(flow.params['state'])
@@ -193,7 +191,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
         # Test extra args specified in the constructor.
         app = flask.Flask(__name__)
         app.config['SECRET_KEY'] = 'notasecert'
-        oauth2 = FlaskOAuth2(
+        oauth2 = flask_util.UserOAuth2(
             app, client_id='client_id', client_secret='secret',
             extra_arg='test')
 
@@ -208,9 +206,9 @@ class FlaskOAuth2Tests(unittest.TestCase):
             q = urlparse.parse_qs(location.split('?', 1)[1])
             state = json.loads(q['state'][0])
 
-            self.assertTrue(GOOGLE_AUTH_URI in location)
-            self.assertFalse(self.oauth2.client_secret in location)
-            self.assertTrue(self.oauth2.client_id in q['client_id'])
+            self.assertIn(oauth2client.GOOGLE_AUTH_URI, location)
+            self.assertNotIn(self.oauth2.client_secret, location)
+            self.assertIn(self.oauth2.client_id, q['client_id'])
             self.assertEqual(
                 flask.session['google_oauth2_csrf_token'], state['csrf_token'])
             self.assertEqual(state['return_url'], '/')
@@ -225,7 +223,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
         with self.app.test_client() as client:
             response = client.get('/oauth2authorize?extra_param=test')
             location = response.headers['Location']
-            self.assertTrue('extra_param=test' in location)
+            self.assertIn('extra_param=test', location)
 
     def _setup_callback_state(self, client, **kwargs):
         with self.app.test_request_context():
@@ -240,7 +238,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
             with client.session_transaction() as session:
                 session.update(flask.session)
                 csrf_token = session['google_oauth2_csrf_token']
-                flow = _get_flow_for_token(csrf_token)
+                flow = flask_util._get_flow_for_token(csrf_token)
                 state = flow.params['state']
 
         return state
@@ -255,9 +253,9 @@ class FlaskOAuth2Tests(unittest.TestCase):
                     '/oauth2callback?state={0}&code=codez'.format(state))
 
                 self.assertEqual(response.status_code, httplib.FOUND)
-                self.assertTrue('/return_url' in response.headers['Location'])
-                self.assertTrue(self.oauth2.client_secret in http.body)
-                self.assertTrue('codez' in http.body)
+                self.assertIn('/return_url', response.headers['Location'])
+                self.assertIn(self.oauth2.client_secret, http.body)
+                self.assertIn('codez', http.body)
                 self.assertTrue(self.oauth2.storage.put.called)
 
     def test_authorize_callback(self):
@@ -273,7 +271,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
 
             response = client.get('/oauth2callback?state={}&error=something')
             self.assertEqual(response.status_code, httplib.BAD_REQUEST)
-            self.assertTrue('something' in response.data.decode('utf-8'))
+            self.assertIn('something', response.data.decode('utf-8'))
 
         # CSRF mismatch
         with self.app.test_client() as client:
@@ -333,9 +331,8 @@ class FlaskOAuth2Tests(unittest.TestCase):
             self.assertTrue(self.oauth2.credentials is None)
             self.assertTrue(self.oauth2.user_id is None)
             self.assertTrue(self.oauth2.email is None)
-            self.assertRaises(
-                ValueError,
-                self.oauth2.http)
+            with self.assertRaises(ValueError):
+                self.oauth2.http()
             self.assertFalse(self.oauth2.storage.get())
             self.oauth2.storage.delete()
 
@@ -352,6 +349,24 @@ class FlaskOAuth2Tests(unittest.TestCase):
             self.assertEqual(self.oauth2.email, 'user@example.com')
             self.assertTrue(self.oauth2.http())
 
+    @mock.patch('oauth2client.client._UTCNOW')
+    def test_with_expired_credentials(self, utcnow):
+        utcnow.return_value = datetime.datetime(1990, 5, 29)
+
+        credentials = self._generate_credentials()
+        credentials.token_expiry = datetime.datetime(1990, 5, 28)
+
+        # Has a refresh token, so this should be fine.
+        with self.app.test_request_context():
+            self.oauth2.storage.put(credentials)
+            self.assertTrue(self.oauth2.has_credentials())
+
+        # Without a refresh token this should return false.
+        credentials.refresh_token = None
+        with self.app.test_request_context():
+            self.oauth2.storage.put(credentials)
+            self.assertFalse(self.oauth2.has_credentials())
+
     def test_bad_id_token(self):
         credentials = self._generate_credentials()
         credentials.id_token = {}
@@ -370,8 +385,8 @@ class FlaskOAuth2Tests(unittest.TestCase):
         with self.app.test_client() as client:
             response = client.get('/protected')
             self.assertEqual(response.status_code, httplib.FOUND)
-            self.assertTrue('oauth2authorize' in response.headers['Location'])
-            self.assertTrue('protected' in response.headers['Location'])
+            self.assertIn('oauth2authorize', response.headers['Location'])
+            self.assertIn('protected', response.headers['Location'])
 
         credentials = self._generate_credentials(scopes=self.oauth2.scopes)
 
@@ -382,13 +397,42 @@ class FlaskOAuth2Tests(unittest.TestCase):
 
             response = client.get('/protected')
             self.assertEqual(response.status_code, httplib.OK)
-            self.assertTrue('Hello' in response.data.decode('utf-8'))
+            self.assertIn('Hello', response.data.decode('utf-8'))
+
+        # Expired credentials with refresh token, should allow.
+        credentials.token_expiry = datetime.datetime(1990, 5, 28)
+        with mock.patch('oauth2client.client._UTCNOW') as utcnow:
+            utcnow.return_value = datetime.datetime(1990, 5, 29)
+
+            with self.app.test_client() as client:
+                with client.session_transaction() as session:
+                    session['google_oauth2_credentials'] = (
+                        credentials.to_json())
+
+                response = client.get('/protected')
+                self.assertEqual(response.status_code, httplib.OK)
+                self.assertIn('Hello', response.data.decode('utf-8'))
+
+        # Expired credentials without a refresh token, should redirect.
+        credentials.refresh_token = None
+        with mock.patch('oauth2client.client._UTCNOW') as utcnow:
+            utcnow.return_value = datetime.datetime(1990, 5, 29)
+
+            with self.app.test_client() as client:
+                with client.session_transaction() as session:
+                    session['google_oauth2_credentials'] = (
+                        credentials.to_json())
+
+                response = client.get('/protected')
+            self.assertEqual(response.status_code, httplib.FOUND)
+            self.assertIn('oauth2authorize', response.headers['Location'])
+            self.assertIn('protected', response.headers['Location'])
 
     def _create_incremental_auth_app(self):
         self.app = flask.Flask(__name__)
         self.app.testing = True
         self.app.config['SECRET_KEY'] = 'notasecert'
-        self.oauth2 = FlaskOAuth2(
+        self.oauth2 = flask_util.UserOAuth2(
             self.app,
             client_id='client_idz',
             client_secret='client_secretz',
@@ -410,7 +454,7 @@ class FlaskOAuth2Tests(unittest.TestCase):
         # No credentials, should redirect
         with self.app.test_client() as client:
             response = client.get('/one')
-            self.assertTrue('one' in response.headers['Location'])
+            self.assertIn('one', response.headers['Location'])
             self.assertEqual(response.status_code, httplib.FOUND)
 
         # Credentials for one. /one should allow, /two should redirect.
@@ -424,14 +468,15 @@ class FlaskOAuth2Tests(unittest.TestCase):
             self.assertEqual(response.status_code, httplib.OK)
 
             response = client.get('/two')
-            self.assertTrue('two' in response.headers['Location'])
+            self.assertIn('two', response.headers['Location'])
             self.assertEqual(response.status_code, httplib.FOUND)
 
             # Starting the authorization flow should include the
             # include_granted_scopes parameter as well as the scopes.
             response = client.get(response.headers['Location'][17:])
-            q = urlparse.parse_qs(response.headers['Location'].split('?', 1)[1])
-            self.assertTrue('include_granted_scopes' in q)
+            q = urlparse.parse_qs(
+                response.headers['Location'].split('?', 1)[1])
+            self.assertIn('include_granted_scopes', q)
             self.assertEqual(
                 set(q['scope'][0].split(' ')),
                 set(['one', 'email', 'two', 'three']))
@@ -483,8 +528,4 @@ class FlaskOAuth2Tests(unittest.TestCase):
             self.oauth2.storage.put(self._generate_credentials())
             self.oauth2.storage.delete()
 
-            self.assertFalse('google_oauth2_credentials' in flask.session)
-
-
-if __name__ == '__main__':  # pragma: NO COVER
-    unittest.main()
+            self.assertNotIn('google_oauth2_credentials', flask.session)
